var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { MsgExecuteContractCompat as MsgExecuteContractInjective, } from "@injectivelabs/sdk-ts";
import { buildTokenId, isNativeCosmWasmDenom, isNativeDenomInjective, } from "../cosmwasm";
import { CHAIN_ID_INJECTIVE, coalesceChainId, hexToUint8Array, parseSmartContractStateResponse, tryNativeToHexString, } from "../utils";
import { fromUint8Array } from "js-base64";
/**
 * Creates attestation message
 * @param tokenBridgeAddress Address of Inj token bridge contract
 * @param walletAddress Address of wallet in inj format
 * @param asset Name or address of the asset to be attested
 * For native assets the asset string is the denomination.
 * For foreign assets the asset string is the inj address of the foreign asset
 * @returns Message to be broadcast
 */
export function attestFromInjective(tokenBridgeAddress, walletAddress, asset) {
    return __awaiter(this, void 0, void 0, function () {
        var nonce, isNativeAsset;
        return __generator(this, function (_a) {
            nonce = Math.round(Math.random() * 100000);
            isNativeAsset = isNativeDenomInjective(asset);
            return [2 /*return*/, MsgExecuteContractInjective.fromJSON({
                    contractAddress: tokenBridgeAddress,
                    sender: walletAddress,
                    exec: {
                        msg: {
                            asset_info: isNativeAsset
                                ? {
                                    native_token: { denom: asset },
                                }
                                : {
                                    token: {
                                        contract_addr: asset,
                                    },
                                },
                            nonce: nonce,
                        },
                        action: "create_asset_meta",
                    },
                })];
        });
    });
}
export var createWrappedOnInjective = submitVAAOnInjective;
/**
 * Returns the address of the foreign asset
 * @param tokenBridgeAddress Address of token bridge contact
 * @param client Holds the wallet and signing information
 * @param originChain The chainId of the origin of the asset
 * @param originAsset The address of the origin asset
 * @returns The foreign asset address or null
 */
export function getForeignAssetInjective(tokenBridgeAddress, client, originChain, originAsset) {
    return __awaiter(this, void 0, void 0, function () {
        var queryResult, parsed, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    return [4 /*yield*/, client.fetchSmartContractState(tokenBridgeAddress, Buffer.from(JSON.stringify({
                            wrapped_registry: {
                                chain: coalesceChainId(originChain),
                                address: fromUint8Array(originAsset),
                            },
                        })).toString("base64"))];
                case 1:
                    queryResult = _a.sent();
                    parsed = parseSmartContractStateResponse(queryResult);
                    return [2 /*return*/, parsed.address];
                case 2:
                    e_1 = _a.sent();
                    return [2 /*return*/, null];
                case 3: return [2 /*return*/];
            }
        });
    });
}
/**
 * Return if the VAA has been redeemed or not
 * @param tokenBridgeAddress The Injective token bridge contract address
 * @param signedVAA The signed VAA byte array
 * @param client Holds the wallet and signing information
 * @returns true if the VAA has been redeemed.
 */
export function getIsTransferCompletedInjective(tokenBridgeAddress, signedVAA, client) {
    return __awaiter(this, void 0, void 0, function () {
        var queryResult, parsed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, client.fetchSmartContractState(tokenBridgeAddress, Buffer.from(JSON.stringify({
                        is_vaa_redeemed: {
                            vaa: fromUint8Array(signedVAA),
                        },
                    })).toString("base64"))];
                case 1:
                    queryResult = _a.sent();
                    parsed = parseSmartContractStateResponse(queryResult);
                    return [2 /*return*/, parsed.is_redeemed];
            }
        });
    });
}
/**
 * Checks if the asset is a wrapped asset
 * @param tokenBridgeAddress The address of the Injective token bridge contract
 * @param client Connection/wallet information
 * @param assetAddress Address of the asset in Injective format
 * @returns true if asset is a wormhole wrapped asset
 */
export function getIsWrappedAssetInjective(tokenBridgeAddress, client, assetAddress) {
    return __awaiter(this, void 0, void 0, function () {
        var hexified, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    hexified = tryNativeToHexString(assetAddress, "injective");
                    return [4 /*yield*/, getForeignAssetInjective(tokenBridgeAddress, client, CHAIN_ID_INJECTIVE, new Uint8Array(Buffer.from(hexified)))];
                case 1:
                    result = _a.sent();
                    if (result === null) {
                        return [2 /*return*/, false];
                    }
                    return [2 /*return*/, true];
            }
        });
    });
}
/**
 * Returns information about the asset
 * @param wrappedAddress Address of the asset in wormhole wrapped format (hex string)
 * @param client WASM api client
 * @returns Information about the asset
 */
export function getOriginalAssetInjective(wrappedAddress, client) {
    return __awaiter(this, void 0, void 0, function () {
        var chainId, response, parsed, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    chainId = CHAIN_ID_INJECTIVE;
                    if (isNativeCosmWasmDenom(chainId, wrappedAddress)) {
                        return [2 /*return*/, {
                                isWrapped: false,
                                chainId: chainId,
                                assetAddress: hexToUint8Array(buildTokenId(chainId, wrappedAddress)),
                            }];
                    }
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, client.fetchSmartContractState(wrappedAddress, Buffer.from(JSON.stringify({
                            wrapped_asset_info: {},
                        })).toString("base64"))];
                case 2:
                    response = _b.sent();
                    parsed = parseSmartContractStateResponse(response);
                    return [2 /*return*/, {
                            isWrapped: true,
                            chainId: parsed.asset_chain,
                            assetAddress: new Uint8Array(Buffer.from(parsed.asset_address, "base64")),
                        }];
                case 3:
                    _a = _b.sent();
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/, {
                        isWrapped: false,
                        chainId: chainId,
                        assetAddress: hexToUint8Array(buildTokenId(chainId, wrappedAddress)),
                    }];
            }
        });
    });
}
/**
 * Submits the supplied VAA to Injective
 * @param tokenBridgeAddress Address of Inj token bridge contract
 * @param walletAddress Address of wallet in inj format
 * @param signedVAA VAA with the attestation message
 * @returns Message to be broadcast
 */
export function submitVAAOnInjective(tokenBridgeAddress, walletAddress, signedVAA) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, MsgExecuteContractInjective.fromJSON({
                    contractAddress: tokenBridgeAddress,
                    sender: walletAddress,
                    exec: {
                        msg: {
                            data: fromUint8Array(signedVAA),
                        },
                        action: "submit_vaa",
                    },
                })];
        });
    });
}
export var redeemOnInjective = submitVAAOnInjective;
/**
 * Creates the necessary messages to transfer an asset
 * @param walletAddress Address of the Inj wallet
 * @param tokenBridgeAddress Address of the token bridge contract
 * @param tokenAddress Address of the token being transferred
 * @param amount Amount of token to be transferred
 * @param recipientChain Destination chain
 * @param recipientAddress Destination wallet address
 * @param relayerFee Relayer fee
 * @param payload Optional payload
 * @returns Transfer messages to be sent on chain
 */
export function transferFromInjective(walletAddress, tokenBridgeAddress, tokenAddress, amount, recipientChain, recipientAddress, relayerFee, payload) {
    if (relayerFee === void 0) { relayerFee = "0"; }
    if (payload === void 0) { payload = null; }
    return __awaiter(this, void 0, void 0, function () {
        var recipientChainId, nonce, isNativeAsset, mk_action, mk_initiate_transfer;
        return __generator(this, function (_a) {
            recipientChainId = coalesceChainId(recipientChain);
            nonce = Math.round(Math.random() * 100000);
            isNativeAsset = isNativeDenomInjective(tokenAddress);
            mk_action = payload
                ? "initiate_transfer_with_payload"
                : "initiate_transfer";
            mk_initiate_transfer = function (info) {
                return payload
                    ? {
                        asset: {
                            amount: amount,
                            info: info,
                        },
                        recipient_chain: recipientChainId,
                        recipient: Buffer.from(recipientAddress).toString("base64"),
                        fee: relayerFee,
                        nonce: nonce,
                        payload: payload,
                    }
                    : {
                        asset: {
                            amount: amount,
                            info: info,
                        },
                        recipient_chain: recipientChainId,
                        recipient: Buffer.from(recipientAddress).toString("base64"),
                        fee: relayerFee,
                        nonce: nonce,
                    };
            };
            return [2 /*return*/, isNativeAsset
                    ? [
                        MsgExecuteContractInjective.fromJSON({
                            contractAddress: tokenBridgeAddress,
                            sender: walletAddress,
                            exec: {
                                msg: {},
                                action: "deposit_tokens",
                            },
                            funds: { denom: tokenAddress, amount: amount },
                        }),
                        MsgExecuteContractInjective.fromJSON({
                            contractAddress: tokenBridgeAddress,
                            sender: walletAddress,
                            exec: {
                                msg: mk_initiate_transfer({
                                    native_token: { denom: tokenAddress },
                                }),
                                action: mk_action,
                            },
                        }),
                    ]
                    : [
                        MsgExecuteContractInjective.fromJSON({
                            contractAddress: tokenAddress,
                            sender: walletAddress,
                            exec: {
                                msg: {
                                    spender: tokenBridgeAddress,
                                    amount: amount,
                                    expires: {
                                        never: {},
                                    },
                                },
                                action: "increase_allowance",
                            },
                        }),
                        MsgExecuteContractInjective.fromJSON({
                            contractAddress: tokenBridgeAddress,
                            sender: walletAddress,
                            exec: {
                                msg: mk_initiate_transfer({
                                    token: { contract_addr: tokenAddress },
                                }),
                                action: mk_action,
                            },
                        }),
                    ]];
        });
    });
}
export var updateWrappedOnInjective = submitVAAOnInjective;
